package pal;

import java.util.Scanner;

// Each function which reads and evaluates a substring of the input string
// must return two values:
// (1) a string/integer value of the substring
// (2) the index where next evaluation will start, i.e the index of the
//  end of the processed substring increased by 1.
// The class Result serves this purpose, thus all relevant functions
// return (1) and (2) simultaneously in a single instance of this class
class Result{
  int num;
  String sg;
  int endIndex;
}

// the program itself
public class Recursive {

static int skipBlanks(String inputSg, int index) {
 while (inputSg.charAt(index) == ' ') index++;
 return index;
}

// read a string constant from the input string and return its value
static Result evalStringConstant(String inputSg, int index){
   Result result = new Result();
   result.sg = "";
   index++;  // skip the opening quotes
   while (inputSg.charAt(index) != '"')
     result.sg = result.sg+inputSg.charAt(index++);
   result.endIndex = index+1; // return the index of the first char past the closing quotes
   return result;
}

// read an integer constant from the input string and return its value
static Result evalIntegerConstant(String inputSg, int index){
   Result result = new Result(); result.sg = "";

   while (('0' <= inputSg.charAt(index)) && (inputSg.charAt(index) <= '9') )
     result.num = result.num*10 + Character.digit(inputSg.charAt(index++), 10);
   result.endIndex = index;
   return result;
}

static String [] variables = new String [26];

// read a variable identifier from the input string and return the value of the variable
static Result evalVariable(String inputSg, int index){
   Result result = new Result();
   result.sg = variables[inputSg.charAt(index)-97];
   result.endIndex = index+1;
   return result;
}

// evaluate the Insert function when the value of its parameters is known
static String functionInsert(String sgWhere, int position, String sgWhat) {
  if (position <= 0)
     return sgWhat + sgWhere;
  if (position < sgWhere.length())
     return sgWhere.substring(0, position)
             + sgWhat + sgWhere.substring(position, sgWhere.length());
  // remains: position >= sgWhere.length()
  return sgWhere + sgWhat;
}

// evaluate the Delete function when the value of its parameters is known
static String functionDelete(String sgWhere, int startPos, int endPos) {
  if (sgWhere.length() == 0)
     return sgWhere;
  if ((endPos < startPos) || (endPos < 0) || (startPos >= sgWhere.length()-1))
     return sgWhere; // odd start and/or end positions, no change to string at all
  if (startPos < 0) startPos = 0;
  if (endPos > sgWhere.length()-1) endPos = sgWhere.length()-1;
  // now the positions are not pathological
  return sgWhere.substring(0, startPos)+ sgWhere.substring(endPos+1, sgWhere.length());
}

// evaluate the Extract function when the value of its parameters is known
static String functionExtract(String sgWhere, int startPos, int endPos) {
  if (sgWhere.length() == 0)
     return sgWhere;
  if ((endPos < startPos) || (endPos < 0) || (startPos >= sgWhere.length()-1))
     return ""; // odd start and/or end positions
  if (startPos < 0) startPos = 0;
  if (endPos > sgWhere.length()-1) endPos = sgWhere.length()-1;
  // now the positions are not pathological
  return sgWhere.substring(startPos, endPos+1);
}

// evaluate the Position function when the value of its parameters is known
static int functionPosition(String sgWhere, String sgWhat) {
  if (sgWhere.length() == 0)
     return -1;
  if (sgWhat.length() == 0)
     return 0;
  return sgWhere.indexOf(sgWhat);
}

// evaluate the Position function when the value of its parameters is known
static int functionLength(String sgOfWhat) {
  return sgOfWhat.length();
}

// evaluate
static Result evalExpression(String inputSg, int index) {
 Result result = new Result();
 result.sg = "";
 int numParam1, numParam2;
 String sgParam1, sgParam2;

 index = skipBlanks(inputSg, index);
 switch (inputSg.charAt(index)) {
   case 'I': {
     // evaluate (recursively) the values of parameters of the Insert function
     result = evalExpression(inputSg, skipBlanks(inputSg,skipBlanks(inputSg,index)+1)+1); // skip over "I("
     sgParam1 = result.sg;

     result = evalExpression(inputSg, skipBlanks(inputSg, result.endIndex)+1); // skip over ','
     numParam1 = result.num;

     result = evalExpression(inputSg, skipBlanks(inputSg, result.endIndex)+1);  // skip over ','
     sgParam2 = result.sg;

     // now just apply the Insert function to the evaluated parameters
     result.sg = functionInsert(sgParam1, numParam1, sgParam2);
     // and skip over the closing bracket ')' to tell the calling proc where to continue
     result.endIndex = skipBlanks(inputSg, result.endIndex)+1;
     break;
   }
   case 'D': {
     // evaluate (recursively) the values of parameters of the Delete function
     result = evalExpression(inputSg, skipBlanks(inputSg,skipBlanks(inputSg,index)+1)+1); // skip over "D("
     sgParam1 = result.sg;

     result = evalExpression(inputSg, skipBlanks(inputSg, result.endIndex)+1); // skip over ','
     numParam1 = result.num;

     result = evalExpression(inputSg, skipBlanks(inputSg, result.endIndex)+1);  // skip over ','
     numParam2 = result.num;

     // now just apply the Delete function to the evaluated parameters
     result.sg = functionDelete(sgParam1, numParam1, numParam2);
     // and skip over the closing bracket ')' to tell the calling proc where to continue
     result.endIndex = skipBlanks(inputSg, result.endIndex)+1;
     break;
   }
   case 'E': {
     // evaluate (recursively) the values of parameters of the Extract function
     result = evalExpression(inputSg, skipBlanks(inputSg,skipBlanks(inputSg,index)+1)+1); // skip over "E("
     sgParam1 = result.sg;

     result = evalExpression(inputSg, skipBlanks(inputSg, result.endIndex)+1); // skip over ','
     numParam1 = result.num;

     result = evalExpression(inputSg, skipBlanks(inputSg, result.endIndex)+1);  // skip over ','
     numParam2 = result.num;

     // now just apply the Extract function to the evaluated parameters
     result.sg = functionExtract(sgParam1, numParam1, numParam2);
     // and skip over the closing bracket ')' to tell the calling proc where to continue
     result.endIndex = skipBlanks(inputSg, result.endIndex)+1;
     break;
   }
   case 'P': {
     // evaluate (recursively) the values of parameters of the Position function
     result = evalExpression(inputSg, skipBlanks(inputSg,skipBlanks(inputSg,index)+1)+1); // skip over "P("
     sgParam1 = result.sg;

     result = evalExpression(inputSg, skipBlanks(inputSg, result.endIndex)+1);  // skip over ','
     sgParam2 = result.sg;

     // now just apply the Position function to the evaluated parameters
     result.num = functionPosition(sgParam1, sgParam2);
     // and skip over the closing bracket ')' to tell the calling proc where to continue
     result.endIndex = skipBlanks(inputSg, result.endIndex)+1;
     break;
   }

   case 'L': {
     // evaluate (recursively) the values of parameters of the Length function
     result = evalExpression(inputSg, skipBlanks(inputSg,skipBlanks(inputSg,index)+1)+1); // skip over "L("
     sgParam1 = result.sg;

     // now just apply the Length function to the evaluated parameters
     result.num = functionLength(sgParam1);
     // and skip over the closing bracket ')' to tell the calling proc where to continue
     result.endIndex = skipBlanks(inputSg, result.endIndex)+1;
     break;
   }


   case '"':  {
     result = evalStringConstant(inputSg, index);
     break;
   }
   case'0': case'1':case '2':case'3':case'4':case'5':case '6':case'7':case'8':case '9': {
     result = evalIntegerConstant(inputSg, index);
     break;
   }
   case'a':case'b':case'c':case'd':case'e':case'f':case'g':case'h':case'i':case'j':
   case'k':case'l':case'm':case'n':case'o':case'p':case'q':case'r':case's':case't':
   case'u':case'v':case'w':case'x':case'y':case'z': {
     result = evalVariable(inputSg, index);
     break;
   }
 }
 return result;
}

static char evalCommand(String inputSg) {
  int index = skipBlanks(inputSg, 0);
  char variableName = inputSg.charAt(index);
  index++; //move over variable name
  Result result = evalExpression(inputSg, skipBlanks(inputSg, index)+1); //  skip the '=' sign
  variables[variableName-97] = result.sg;
  return variableName;
}

static void evalProgram() {
  Scanner scanner = new Scanner(System.in);
  String inputLine;
  char variableName;
  do {
    inputLine = scanner.nextLine();
    variableName = evalCommand(inputLine);
  } while (inputLine.indexOf('(') == -1);
  System.out.printf("\"%s\"\n", variables[variableName-97]);
}

  public static void main(String[] args) {
    evalProgram();
  }
}