package pal;

import java.util.Scanner;

// Each function which reads and evaluates a substring of the input string
// must return two values:
// (1) a string/integer value of the substring
// (2) the index where next evaluation will start, i.e the index of the
//  end of the processed substring increased by 1.
// The class Result serves this purpose, thus all relevant functions
// return (1) and (2) simultaneously in a single instance of this class

class Result{
  int num;
  String sg;
  int endIndex;
}

public class RecursiveBasic {

// read a string constant from the input string and return its value
static Result evalStringConstant(String inputSg, int index){
   Result result = new Result();
   result.sg = "";
   index++; // skip the opening quotes
   while (inputSg.charAt(index) != '"')
     result.sg = result.sg + inputSg.charAt(index++);
   result.endIndex = index+1; // return the index of the first char past the closing quotes
   return result;
}

// read an integer constant from the input string and return its value
static Result evalIntegerConstant(String inputSg, int index){
   Result result = new Result(); result.sg = "";
   while (('0' <= inputSg.charAt(index)) && (inputSg.charAt(index) <= '9') )
     result.num = result.num*10 + Character.digit(inputSg.charAt(index++), 10);
   result.endIndex = index;
   return result;
}

// evaluate the Insert function when the value of its parameters is known
static String functionInsert(String sgWhere, int position, String sgWhat) {
  if (position <= 0)
     return sgWhat + sgWhere;
  if (position < sgWhere.length())
     return sgWhere.substring(0, position)
             + sgWhat + sgWhere.substring(position, sgWhere.length());
  // remains: position >= sgWhere.length()
  return sgWhere + sgWhat;
}

// evaluate an expression stored in a substring of the input string
static Result evalExpression(String inputSg, int index) {
 Result result = new Result();
 result.sg = "";
 int numParam1;
 String sgParam1, sgParam2;

 switch (inputSg.charAt(index)) {
   case 'I':{
     // evaluate (recursively) the values of parameters of the Insert function
     result = evalExpression(inputSg, index+2); // index+2 means skip over "I("
     sgParam1 = result.sg;
     result = evalExpression(inputSg, result.endIndex+1);   //index+1 means skip over ','
     numParam1 = result.num;
     result = evalExpression(inputSg, result.endIndex+1);   //index+1 means skip over ','
     sgParam2 = result.sg;
     // now just apply the Insert function to the evaluated parameters
     result.sg = functionInsert(sgParam1, numParam1, sgParam2);
     /// and skip over the closing bracket ')' to tell the calling proc where to continue
     result.endIndex++;
     break;
   }
   case '"':  {
     result = evalStringConstant(inputSg, index);
     break;
   }
   case'0': case'1':case '2':case'3':case'4':case'5':case '6':case'7':case'8':case '9': {
     result = evalIntegerConstant(inputSg, index);
     break;
   }
 }
 return result;
}

  public static void main(String[] args) {
    Scanner scanner = new Scanner(System.in);
    System.out.printf("\"%s\"\n", evalExpression(scanner.nextLine(), 2).sg);
  }
}
